;; Generated package description from greader.el  -*- no-byte-compile: t; lexical-binding:t -*-
(define-package "greader" "0.17.0.0.20260324.123610" "Gnamù reader, send buffer contents to a speech engine" '((emacs "26.1") (seq "2.24") (compat "29.1.4.5")) :commit "4fd1ae83d21f6349cd849f3652e7bf8f00edc9dd" :authors '(("Michelangelo Rodriguez" . "michelangelo.rodriguez@gmail.com")) :maintainer '("Michelangelo Rodriguez" . "michelangelo.rodriguez@gmail.com") :keywords '("tools" "accessibility") :url "https://gitlab.com/michelangelo-rodriguez/greader")
